@@ -6,7 +6,7 @@ Toadfarm - One Mojolicious app to rule them all
 
 =head1 VERSION
 
-0.53
+0.54
 
 =head1 DESCRIPTION
 
@@ -153,7 +153,7 @@ use File::Spec;
 use File::Which;
 use constant DEBUG => $ENV{TOADFARM_DEBUG} ? 1 : 0;
 
-our $VERSION = '0.53';
+our $VERSION = '0.54';
 
 BEGIN {
   $ENV{TOADFARM_ACTION} //= $ENV{MOJO_APP_LOADER} ? 'load' : (@ARGV and $ARGV[0] =~ /^(reload|start|stop)$/) ? $1 : '';
@@ -179,8 +179,9 @@ sub import {
     logging => sub { $got{log}++; $app->_setup_log(@_) },
     mount   => sub { push @{$app->config->{apps}}, @_ == 2 ? @_ : ($_[0], {}); $app },
     plugin  => sub { push @{$app->config->{plugins}}, @_ == 2 ? @_ : ($_[0], {}); $app },
+    run_as  => \&_run_as,
     secrets => sub { $got{secrets} = 1; $app->secrets([@_]) },
-    start => sub {
+    start   => sub {
       if (@_) {
         my $listen = ref $_[0] eq 'ARRAY' ? shift : undef;
         $app->config->{hypnotoad} = @_ > 1 ? {@_} : {%{$_[0]}} if @_;
@@ -193,6 +194,7 @@ sub import {
       $app = $class->_setup_app($app, \%got) if $action eq 'load';
       Mojo::UserAgent::Server->app($app);
       warn '$config=' . Mojo::Util::dumper($app->config) if DEBUG;
+      $class->_die_on_insecure($app) unless $ENV{TOADFARM_INSECURE};
       $app->start;
     },
   );
@@ -215,6 +217,14 @@ sub startup {
   $self->_mount_root_app(@{delete $self->{root_app}}) if $self->{root_app};
 }
 
+sub _die_on_insecure {
+  my ($class, $app) = @_;
+
+  die "Cannot change user without TOADFARM_INSECURE=1"   if $app->config->{hypnotoad}{user};
+  die "Cannot change group without TOADFARM_INSECURE=1"  if $app->config->{hypnotoad}{group};
+  die "Cannot run as 'root' without TOADFARM_INSECURE=1" if $> == 0 or $< == 0;
+}
+
 sub _exit { say shift and exit 0 }
 
 sub _load_plugins {
@@ -328,6 +338,30 @@ sub _pid_file {
   return File::Spec->catfile(File::Spec->tmpdir, "toadfarm-$name.pid");
 }
 
+sub _run_as {
+  my $user = shift || die "Usage: run_as('username')";
+  my ($exit, $uid, @sudo);
+
+  $uid = $user =~ m!^\d+$! ? $user : scalar getpwnam $user;
+  die "Could not find uid for user $user\n" unless $uid;
+  return 1 if $uid == $>;
+
+  for my $p (File::Spec->path) {
+    $sudo[0] = File::Spec->catfile($p, 'sudo');
+    next unless -x $sudo[0];
+    push @sudo, '-n', -u => "#$uid", $^X;
+    last;
+  }
+
+  die "Cannot change to uid=$uid: 'sudo' was not found.\n" unless @sudo > 1;
+  push @sudo, -I => $INC[0] if $ENV{TOADFARM_ACTION} eq 'test';
+  push @sudo, File::Spec->rel2abs($0), @ARGV;
+  warn "[Toadfarm] system @sudo\n" if DEBUG;
+  system @sudo;
+  die "Could not run '@sudo' exit=$exit\n" if $exit = $? >> 8;
+  exit $?;
+}
+
 sub _setup_app {
   my ($class, $app, $got) = @_;
   my $config = $app->config;
